"""
Модуль предназначен для эквивалентирования
"""

# import Python lib
import logging

# import win32com
from win32com.client import CDispatch

logger = logging.getLogger("PyRastrWin")


def ekv(rastr_win: CDispatch, par: str = "") -> bool:
    """
    Эквивалентирование режима.

    :param rastr_win: Dispatch("Astra.Rastr").
    :param par: строка дополнительных параметров.
        Параметры могут быть следующими:
            "" – c параметрами по умолчанию;
            "p" – расчет с плоского старта;
            "z" – отключение стартового алгоритма;
            "c" – отключение контроля данных;
            "r" – отключение подготовки данных (можно использовать при повторном расчете режима с измененными
                  значениями узловых мощностей и модулей напряжения).
    :return None
    """
    logger.info("Эквивалентирование запущенно")
    params = {
        "": ["", "c параметрами по умолчанию"],
        "p": ["p", "p - расчет с плоского старта"],
        "z": ["z", "z - отключение стартового алгоритма"],
        "c": ["c", "c -  отключение контроля данных"],
        "r": [
            "r",
            "r - отключение подготовки данных (можно использовать при повторном расчете режима с измененными "
            "значениями узловых мощностей и модулей напряжения)",
        ],
    }
    try:
        kod = rastr_win.ekv(params[par])
    except KeyError:
        par = ""
        kod = rastr_win.ekv(params[par])
    if kod == 0:
        logger.info(f"Режим сбалансирован: %s", params[par][1])
        return True
    else:
        logger.warning(f"Режим не сбалансирован: %s", params[par][1])
        return False
