"""
Настройки параметров расчета динамики.

----------------------------------------------------------------
Навигация UI: Расчеты -> Параметры -> Таблица: Динамика
----------------------------------------------------------------

>>  Время расчета (Т_расч) in code (Tras): – задает длительность расчета ЭМПП в секундах. Т_расч ~ 10 с.

>>  Начальный шаг интегрирования (H_инт) in code (Hint) – шаг интегрирования для методов с постоянным шагом. Для методов
    с переменным шагом не используется. Используется только в методах: H_инт = 0,001.

>>  Минимальный и максимальный шаг интегрирования (H_макс и H_мин) in code (Hmin, Hmax) – используется для методов
    интегрирования с переменным шагом. При возникновении в модели события, которое вызывает необходимость рестарта метода
    интегрирования (отключение, включение линии, работа дискретной автоматики), все методы начинают работу с минимального шага.
    Можно установить значение максимального шага в значение Hmax = 1-5 с.
    Можно установить значение максимального шага в значение Hmin = 0.0001 с.
    H_мин = 0,001 с.
    H_макс = 5 с.

>>  Шаг печати (H_печ) in code (Hout)–для вывода результатов на график и отображения результатов расчета в режимном файле.
    H_печ = 0,001 с.

>>  Основной метод интегрирования (Осн.Метод) in code (Mint) – позволяют выбрать один из встроенных методов. Подробная
    информация о методах интегрирования приведена в разделе 4.2.
    0: НН5 - метод Хайхэма-Холла 5 порядка (обеспечивает автоматический выбор шага интегрирования);
    1: PK4 - метод Рунге-Кутты 4 порядка (не обеспечивает автоматический выбор шага интегрирования);
    2: KM4 - метод Кутты-Мерсона 4 порядка (обеспечивает автоматический выбор шага интегрирования);
    3: DP5 - метод Дормана-Принса 5 порядка (обеспечивает автоматический выбор шага интегрирования);
    4: BDF4 - метод обратного дифференцирования 4 порядка (обеспечивает автоматический выбор шага интегрирования);
    5: Adams4 - метод Адамса-Моултона 4 порядка (обеспечивает автоматический выбор шага интегрирования);
    6: НН5w - метод Хайхэма-Холла 5 порядка (обеспечивает автоматический выбор шага интегрирования).
    Методы HH5 и DP5 являются наилучшими из реализованных и рекомендуются к применению.
    Осн.Метод = "НН5" (0).

>>  Стартовый метод интегрирования (Стартовый.Метод) in code (SMint) – позволяют выбрать один из встроенных методов.
    Подробная информация о методах интегрирования приведена в разделе 4.2.
    0: KM4 - метод Кутты-Мерсона 4 порядка (обеспечивает автоматический выбор шага интегрирования);
    2: DP5 - метод Дормана-Принса 5 порядка (обеспечивает автоматический выбор шага интегрирования);
    3: НН5 - метод Хайхэма-Холла 5 порядка (обеспечивает автоматический выбор шага интегрирования);
    В качестве стартовых могут быть использованы все имеющиеся в ПК RUSTab методы, кроме собственно Адамс4 и BDF4.
    Рекомендуется метод КМ4. В процессе модернизации ПК RUSTab многошаговые методы были исследованы и признаны
    недостаточно эффективными. Поэтому они размещены в конце списка основных методов интегрирования.
    Их применение для расчетов ЭМПП не рекомендуется.
    Стартовый.Метод = "КМ4" (0)

>>  Точность шага интегрирования (dInt) in code (IntEpsilon) - Точность шага интегрирования по умолчанию установлена в
    значение 10^−5 (0.00001), что обеспечивает адекватную погрешность при расчете подавляющего большинства ЭМПП.
    Для ускорения расчета можно увеличивать этот параметр до тех пор, пока результат расчета не будет существенно отличаться
    от эталонного, полученного при точности 10^−5 (0.00001). dInt = 0.00001.

>>  Информировать об изменении шага (Выводить шаг) in code (InformOnStepChange) – переключатель, который управляет
    отображением сообщений о смене шага.
    0 - откл. (убрать галочку); 1 - вкл. (поставить галочку)
    При использовании метода с переменным шагом предусмотрена информация о выборе шага, которая выводится в протокол.
    При длительном расчете сообщений об изменении шага может выводиться довольно много, что загромождает протокол.
    Есть возможность отключить эти сообщения.
    Одновременно отключаются сообщения о переменной, из-за нарушения точности расчета которой шаг уменьшается.
    Выводить шаг = 0 - откл. (убрать галочку).

>>  Постоянная сглаживания угловой скорости (частоты) узла (Tf) in code (Tf) – постоянная времени реально
    дифференцирующего звена для вычисления 𝑆𝑢=(𝑝(1+𝑝𝑇𝑓)⁄)𝛿𝑈.
    Tf=0.02 (По умолчанию).

>>  Точность балансировки эдс при учете явнополюсности (dEf) in code (dEf) – используется при учете явнополюсности.
    dEf=0,4 (По умолчанию).

>>  Макс число пересчетов УР на шаге при учете явнополюсности (Ит) in code (Npf) – ограничивает число решений системы
    линейных уравнений для учета нелинейности нагрузки и явнополюсности. Ит=2000-5000.
    Ит=5000.
    Параметр «Максимальное число пересчетов УР при учете явнополюсности» имеет не совсем удачное название,
    но сохранен с первых версий ПК RUSTab. Параметр вообще не имеет отношения к расчету УР и лишь косвенно связан
    с учетом распределения токов в явнополюсной машине. На самом деле этот параметр предназначен для решения
    системы алгебраических уравнений, которое метод интегрирования выполняет в процессе решения
    дифференциально-алгебраической системы. Поскольку модель ЭМПП нелинейна, выполнение шага интегрирования
    требует решения нелинейной системы. В ПК RUSTab такое решение выполняется по методу простой итерации.
    Сходимость такого решения в большинстве случаев удовлетворительная, так как изменения параметров между
    шагами интегрирования ограничены. Но когда в модели присутствуют элементы со значительной нелинейностью,
    количество требуемых для решения итераций может достигать нескольких тысяч. В частности, такой эффект наблюдается
    при расчете ЭМПП с учетом моделей силовой электроники. Поэтому, если в протоколе расчета появляется сообщение о
    недостаточном количестве итераций при решении системы уравнений сети, можно увеличивать этот параметр
    вплоть до значений 2000-5000.
    Это резко снизит скорость расчета, но позволит, по крайней мере, попытаться рассчитать такой ЭМПП.

>>  Контроль входных параметров (Контр.) in code (Valid) – выполняется либо «Строго», когда расчет не начинается при
    неверно заданных параметрах (например, 𝑋𝑑′ >𝑋𝑑), либо «По умолчанию» – в этом случае неверно заданные параметры
    заменяются на заданные по умолчанию.
    0 - По умолчанию;
    1 - Строго.
    Контр. = 0 (По умолчанию).

>>  Демпфирование в уравнениях движения (Демпф.) in code (dempfrec) – коэффициент демпфирования умножается либо на
    S (скольжение генератора), либо на S-Su (разность скольжений узла и генератора).
    0 - S;
    1 - S-Su.
    Демпф. = 0 (S).

>>  Корректировать Т в парковских моделях (Корр Т) in code (corrT) – определяет способ расчета постоянных времени в
    трех- и четырехконтурных моделях Парка:
    «Да» – используется точная методика (предложена НИИПТ, требует точного задания параметров модели, т.к. при неточном
     может приводить к аварийной остановке из-за невозможности вычислить квадратный корень из отрицательного числа);
    «Нет» – используется классический алгоритм вычисления постоянных времени (математика описана в
    «Моделях генераторов» 5.1).
    0 - Да;
    1 - Нет.
    Корр Т=1 (Нет)

>>  Учет демп. момента в моделях с демпферными контурами (Уч Демп) in code (IsDemp) – определяет, учитывать или нет
    заданный коэффициент демпфирования в моделях генератора, имеющих демпферные контуры; по умолчанию «Нет».
    0 - Нет;
    1 - Да.
    Корр Т=1 (Да)

>>  Напряжения перехода с СХН на шунт (V_минСХРН) in code (frSXNtoY) – минимальное напряжение в о.е. с заданной
    статической характеристикой шунта.
    V_минСХРН = 1.000 (По умолчанию)

>>  Допустимый небаланс СХН (SXNTol) in code (SXNTolerance) - ???.
    SXNTol = 0 (По умолчанию).

>>  Выходной каталог файлов результатов (Кат. результатов) in code (SnapPath)  – путь к каталогу, в котором сохраняются
    файлы результатов расчетов. Если путь к этому каталогу не существует, будет предпринята попытка создать его
    при условии, что все элементы пути не имеют защиты от записи.
    Кат. результатов = "c:\tmp\" (По умолчанию).

>>  Максимальное кол-во файлов результатов (Макс. файлов) in code (MaxResultFiles) – максимальное количество файлов
    результатов, которые могут быть автоматически загружены в элемент управления «Файлы расчетов».
    При нулевом значении ограничение не действует. При значении, отличном от нуля, загрузка файла, порядковый номер
    которого превышает значение ограничения, приведет к удалению из элемента управления первого загруженного файла.
    На количество результатов в списке может быть наложено ограничение с помощью данного параметра.
    Если после выполнения расчета количество результатов равно заданному в этом параметре – первый
    (самый ранний результат) удаляется из списка, и список сдвигается вверх, чтобы добавить новый результат.
    Параметр «Фикс» позволяет запретить удаление результата. При этому будут удаляться результаты без отметки в этой
    колонке в порядке от первого до последнего. Если все результаты будут помечены в колонке «Фикс»,
    и количество результатов будет больше или равно заданному максимальному количеству результатов, новые результаты
    добавляться в список не будут. Файлы результатов по-прежнему будут записываться в указанный выходной каталог и их
    можно будет загрузить вручную.
    Макс. файлов = 0 (По умолчанию).

>>  Шаблон имени выходного файла (Шаблон имени) in code (SnapTemplate) - В колонке «Файл» отображаются имена файлов
    загруженных результатов. Имена формируются при расчете автоматически по шаблону, заданному в параметрах расчета
    динамики (см. раздел 4.1), и отличаются временем выполнения расчета или порядковым номеров. При необходимости
    наиболее выдающимся результатам можно присвоить особенное, информативное и легко читаемое имя.
    Например, «ОЭС-Юж.Судана-2016-Зим.макс-1КЗ-Дарфур-УРОВ-220-Кассала-ТАПВ-Гедареф-Сеннар-2.1с».
    Для этого колонка «Файл» поддерживает режим редактирования имени файла.
    Пользователь может редактировать как имя файла в текущем пути, так и просто задать имя, удалив текущий путь
    полностью. При переименовании будет изменено только имя файла; путь и расширение будут сохранены.
    Таким образом, с помощью переименования нельзя перенести файл из текущего каталога в другой.
    Например, текущий путь к файлу результата «c:\tmp\0000222.sna».
    Пользователь может задать «c:\tmp\АХ Выдрино-Мымрино.sna», «c:\\MyFiles\\АХ Выдрино-Мымрино» или просто
    «АХ Выдрино-Мымрино».
    Во всех случаях файл «c:\tmp\0000222.sna» будет переименован в «c:\tmp\АХ Выдрино-Мымрино.sna».
    Шаблон имени = "<count>.sna" (По умолчанию).

>>  Автозагрузка последнего результата (Автозагрузка) in code (SnapAutoLoad) - переключатель, который определяет, будет ли
    после успешного завершения расчета загружен соответствующий файл результатов в элемент управления «Файлы расчетов».
    Автозагрузка = 1 (По умолчанию).

>>  Максимальное кол-во слотов результатов (Макс. рез-тов) in code (SnapMaxCount) - максимальное количество файлов
    результатов, которые могут быть автоматически загружены в элемент управления «Файлы расчетов».
    При нулевом значении ограничение не действует. При значении, отличном от нуля, загрузка файла, порядковый номер
    которого превышает значение ограничения, приведет к удалению из элемента управления первого загруженного файла.
    Макс. рез-тов = 6 (По умолчанию).

>>  Отключать генератор при превышении скорости % (Уставка автоматов безопасности) in code (TripGeneratorOnSpeed) -
    задает уставку автоматов безопасности, которые контролируют скорости вращения роторов генераторов.
    Возможность моделирования действия автоматов безопасности турбин для всех генераторов.
    При срабатывании автомата безопасности генератор отключается от сети. Переходный процесс в генераторе после
    отключения не моделируется. Уставка автомата безопасности задается в параметрах расчета в поле
    «Отключить генератор при превышении скорости, % (Уставка автоматов безопасности)» в процентах от номинальной
    скорости вращения. В случае если в поле задано нулевое значение, автоматы безопасности в процессе расчета не
    срабатывают. Если задано ненулевое значение, все генераторы, для которых в процессе расчета переходного процесса
    скорость вращения ротора превысит данное значение, будут отключены. При отключении генератора в протокол будет
    выведено предупреждение с указанием номера генератора и причины отключения.
    Уставка автоматов безопасности = 0 (По умолчанию).

>>  Информировать о пуске/возврате автоматики (Информировать о пуске/возврате автоматики) in code (PickupDropout) -
    позволяет включать или отключать режим протоколирования пусков, возвратов и срабатываний элементов логики,
    а также моделей реле, которые использованы в этих элементах. Сообщения об изменении состояния элементов автоматики
    в протоколе содержат информацию об идентификаторе и названии элемента логики и времени изменения его состояния.
    Информировать о пуске/возврате автоматики = 1 (По умолчанию).

>>  Выводить контролируемые величины в CSV (Выводить контролируемые величины в CSV) in code (RealtimeCSV) - в процессе
    расчета возможен вывод численных значений в сsv-файл. В случае, если опция включена, в каталог, заданный для файлов
    результатов в процессе расчета записывается также csv-файл, с именем, идентичным имени файла результатов,
    но с расширением *.csv.
    Выводить контролируемые величины в CSV = 0 (По умолчанию).

>>  Отображать углы в диапазоне +/-180 (Отображать углы в диапазоне +/-180) in code (PeriodAngle) - включает или
    выключает нормализацию отображаемых на графиках углов. В положении «Нет» угол отображается в виде непрерывного
    графика в диапазоне (−∞; ∞). В положении «Да» угол нормализуется в диапазоне [−180; 180). Опция влияет только
    на отображение результатов расчетов параметров, единицами измерений которых являются электрические или
    механические градусы. Опция не влияет на отображение графиков реального времени.
    0 - Нет;
    1 - Да;
    Отображать углы в диапазоне +/-180 = 1 (Да)

>>  Положительное направление результатов (Положительное направление результатов) in code (ResultFlowDirection) -
    позволяет выбрать режим отображения параметров, рассчитываемых с учетом положительного направления:
    мощности, сопротивления и угла мощности по связи. По умолчанию в RastrWin3 используется положительное направление к
    узлу начала ветви. В режиме отображения «RastrWin3» значения перечисленных параметров при отображении на графиках,
    экспорте в буфер обмена, сsv-файлы и формат COMTRADE (раздел 3.8) соответствуют значениям в таблице «Ветви».
    Для анализа работы устройств РЗА может быть использован одноименный режим отображения, в котором за положительное
    направление принимается направление от шин в линию. Данный режим распространяется только на отображение сохраненных
    результатов расчета, их экспорт в csv-файлы и экспорт в формате COMTRADE, но не оказывает влияния на графики
    реального времени и процесс расчета ЭМПП.
    0 - RastrWin;
    1 - РЗА.
    Положительное направление результатов = 0 (RastrWin).

>> Считать предупреждения ошибками (Предупреждение=Ошибка) in code (TreatWarningsAsErrors) - позволяет включить режим
    замены типа «Предупреждение» на тип «Ошибка» для всех сообщений в протоколе; по умолчанию «Нет».
    0 - Нет;
    1 - Да;
    Предупреждение=Ошибка = 0 (Нет).

>> Метод обработки дискретных изменений (Дискретные изменения) in code (EventProcess) - ????
    0 - Стандартный;
    1 - Расширенный;
    Дискретные изменения = 0 (Стандартный)
"""

# import Python
import logging
from dataclasses import dataclass
from enum import Enum

# import win32com
from win32com.client import CDispatch

logger = logging.getLogger("PyRastrWin")


class Mint(Enum):
    """
    Все основные методы интегрирования
    """

    HH5 = (0, "HH5")
    PK4 = (1, "РК4")
    KM4 = (2, "KM4")
    DP5 = (3, "DP5")
    BDF4 = (4, "BDF4")
    Adams4 = (5, "Adams4")
    HH5w = (6, "HH5w")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class SMint(Enum):
    """
    Все стартовые методы интегрирования
    """

    KM4 = (0, "KM4")
    DP5 = (1, "DP5")
    HH5 = (2, "HH5")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class OutputParameterControl(Enum):
    """
    Контроль выхожных параметров.
    """

    default = (0, "По умолчанию")
    strictly = (1, "Строго")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class Dampf(Enum):
    """
    Демпфирование в уравнениях движения (Демпф)
    """

    s = (0, "S")
    s_su = (0, "S-Su")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class KorrT(Enum):
    """
    Корректировать Т в парковских моделях (Корр Т)
    """

    yes = (0, "Да")
    no = (1, "Нет")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class IsDemp(Enum):
    """
    Учет демп. момента в моделях с демп контурами (Уч Демп)
    """

    yes = (0, "Да")
    no = (1, "Нет")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class SnapPath:
    """
    Выходной каталог файлов результатов (Кат. результатов)
    """

    disk_c = r"C:\tmp"
    disk_d = r"D:\tmp"
    disk_u = r"U:\tmp"
    disk_e = r"E:\tmp"


class SnapTemplate:
    """
    Шаблон имени выходного файла (Шаблон имени)
    """

    default = "<count>.sna"
    count_name_rst_dfw_scn = "<count>_<RSTNAME>_<DFWNAME>_<SCNNAME>.sna"


class PeriodAngle(Enum):
    """
    Отображать углы в диапазоне +/-180 (Отображать углы в диапазоне +/-180)
    """

    no = (0, "Нет")
    yes = (1, "Да")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class ResultFlowDirection(Enum):
    """
    Положительное направление результатов (Положительное направление результатов)
    """

    RastrWin = (0, "RastrWin")
    RZA = (1, "РЗА")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class TreatWarningsAsErrors(Enum):
    """
    Считать предупреждения ошибками (Предупреждение=Ошибка)
    """

    no = (0, "Нет")
    yes = (0, "Да")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class EventProcess(Enum):
    """
    Метод обработки дискретных изменений (Дискретные изменения)
    """

    standard = (0, "Стандартный")
    extended = (1, "Расширенный")

    def __init__(self, num, label):
        self.num = num
        self.label = label


class OOSBlockType(Enum):
    """
    Вид блокировки фиксации АР по напряжению (Блокировка АР)
    """

    absent = (0, "Отсутствует")
    one_of_the_nodes = (1, "Один из узлов")
    both_nodes = (2, "Оба узла")

    def __init__(self, num, label):
        self.num = num
        self.label = label


@dataclass(frozen=False, slots=False)
class Dynamic:
    """
    Расчеты -> Параметры -> Динамика

    Общие данные для расчета динамики.

    :param tras: Время расчета (Т_расч) задает длительность расчета ЭМПП в секундах
    :param hint: Начальный шаг интегрирования (H_инт)
    :param hmin: Минимальный шаг интегрирования (H_мин)
    :param hmax: Максимальный шаг интегрирования (H_макс) ~ 1-5 с.
    :param hout: Шаг печати (H_печ)
    :param mint: НН5 Основной метод интегрирования (Осн.Метод)
    :param smint: КМ4 Стартовый метод интегрирования (Старт.Метод)
    :param int_epsilon: Точность шага интегрирования (dInt)
    :param inform_on_step_change: Информировать об изменении шага (Выводить шаг)
    :param tf: Постоянная сглаживания угловой скорости (частоты) узла (Tf)
    :param dEf: Точность балансировки эдс при учете явнополюсности (dEf)
    :param npf: Макс число пересчетов УР на шаге при учете явнополюсности (Ит)
    :param valid: Контроль входных параметров (Контр.)
    :param dempfrec: Демпфирование в уравнениях движения (Демпф)
    :param corrT: Корректировать Т в парковских моделях (Корр Т)
    :param isDemp: Учет демп. момента в моделях с демп контурами (Уч Демп)
    :param frSXNtoY: Напряжения перехода с СХН на шунт (V_минСХРН)
    :param sxn_tolerance: Допустимый небаланс СХН (SXNTol)
    :param snap_path: Выходной каталог файлов результатов (Кат. результатов)
    :param max_result_files: Максимальное кол-во файлов результатов (Макс. файлов)
    :param snap_template: Шаблон имени выходного файла (Шаблон имени)
    :param snap_auto_load: Автозагрузка последнего результата (Автозагрузка)
    :param snap_max_count: Максимальное кол-во слотов результатов (Макс. рез-тов)
    :param trip_generator_on_speed: Отключать генератор при превышении скорости % (Уставка автоматов безопасности)
    :param pickup_dropout: Информировать о пуске/возврате автоматики
    :param real_time_csv: Выводить контролируемые величины в CSV
    :param period_angle: Отображать углы в диапазоне +/-180
    :param result_flow_direction: Положительное направление результатов
    :param treat_warnings_as_errors: Считать предупреждения ошибками (Предупреждение=Ошибка)
    :param event_process: Метод обработки дискретных изменений (Дискретные изменения)
    :param oos_block_type: Вид блокировки фиксации АР по напряжению (Блокировка АР)
    :param oos_block_voltage: Напряжение блокировки фиксации АР (о.е.) (Vблок АР)
    """

    tras: float = 10.0
    hint: float = 0.001
    hmin: float = 0.001
    hmax: float = 5.00
    hout: float = 0.02
    mint: Mint = Mint.HH5.num
    smint: SMint = SMint.DP5.num
    int_epsilon: float = 0.000001
    inform_on_step_change: bool = True
    tf: float = 0.04
    dEf: float = 0.01
    npf: int = 5000
    valid: OutputParameterControl = OutputParameterControl.default.num
    dempfrec: Dampf = Dampf.s.num
    corrT: KorrT = KorrT.yes.num
    isDemp: IsDemp = IsDemp.yes.num
    frSXNtoY: float = 0.750
    sxn_tolerance: float = 0.001
    snap_path: SnapPath = SnapPath.disk_c
    max_result_files: int = 100
    snap_template: SnapTemplate = SnapTemplate.default
    snap_auto_load: bool = True
    snap_max_count: int = 6
    trip_generator_on_speed: float = 0.0
    pickup_dropout: bool = False
    real_time_csv: bool = False
    period_angle: PeriodAngle = PeriodAngle.no.num
    result_flow_direction: ResultFlowDirection = ResultFlowDirection.RastrWin.num
    treat_warnings_as_errors: TreatWarningsAsErrors = TreatWarningsAsErrors.no.num
    event_process: EventProcess = EventProcess.standard.num
    oos_block_type: OOSBlockType = OOSBlockType.absent.num
    oos_block_voltage: float = 0.0


def settings_for_dynamic(rastr_win: CDispatch, settings: Dynamic = Dynamic()) -> None:
    """
    Выставляет рекомендованные настройки "Динамики".
    :param settings: Dynamic()
    :param rastr_win: Объект RastrWin3.
    :return: None
    """
    logger.info("Выставляет рекомендованные настройки Динамики")
    tb_com_dynamics = rastr_win.Tables("com_dynamics")
    tb_com_dynamics.Cols("Tras").SetZN(0, settings.tras)
    tb_com_dynamics.Cols("Hint").SetZN(0, settings.hint)
    tb_com_dynamics.Cols("Hmin").SetZN(0, settings.hmin)
    tb_com_dynamics.Cols("Hmax").SetZN(0, settings.hmax)
    tb_com_dynamics.Cols("Hout").SetZN(0, settings.hout)
    tb_com_dynamics.Cols("Mint").SetZN(0, settings.mint)
    tb_com_dynamics.Cols("SMint").SetZN(0, settings.smint)
    tb_com_dynamics.Cols("IntEpsilon").SetZN(0, settings.int_epsilon)
    tb_com_dynamics.Cols("InformOnStepChange").SetZ(0, settings.inform_on_step_change)
    tb_com_dynamics.Cols("Tf").SetZ(0, settings.tf)
    tb_com_dynamics.Cols("dEf").SetZ(0, settings.dEf)
    tb_com_dynamics.Cols("Npf").SetZ(0, settings.npf)
    tb_com_dynamics.Cols("Valid").SetZ(0, settings.valid)
    tb_com_dynamics.Cols("dempfrec").SetZ(0, settings.dempfrec)
    tb_com_dynamics.Cols("corrT").SetZ(0, settings.corrT)
    tb_com_dynamics.Cols("IsDemp").SetZ(0, settings.isDemp)
    tb_com_dynamics.Cols("frSXNtoY").SetZ(0, settings.frSXNtoY)
    tb_com_dynamics.Cols("SXNTolerance").SetZ(0, settings.sxn_tolerance)
    tb_com_dynamics.Cols("SnapPath").SetZ(0, settings.snap_path)
    tb_com_dynamics.Cols("MaxResultFiles").SetZ(0, settings.max_result_files)
    tb_com_dynamics.Cols("SnapTemplate").SetZ(0, settings.snap_template)
    tb_com_dynamics.Cols("SnapAutoLoad").SetZ(0, settings.snap_auto_load)
    tb_com_dynamics.Cols("SnapMaxCount").SetZ(0, settings.snap_max_count)
    tb_com_dynamics.Cols("TripGeneratorOnSpeed").SetZ(
        0, settings.trip_generator_on_speed
    )
    tb_com_dynamics.Cols("PickupDropout").SetZ(0, settings.pickup_dropout)
    tb_com_dynamics.Cols("RealtimeCSV").SetZ(0, settings.real_time_csv)
    tb_com_dynamics.Cols("PeriodAngle").SetZ(0, settings.period_angle)
    tb_com_dynamics.Cols("ResultFlowDirection").SetZ(0, settings.result_flow_direction)
    tb_com_dynamics.Cols("TreatWarningsAsErrors").SetZ(
        0, settings.treat_warnings_as_errors
    )
    tb_com_dynamics.Cols("EventProcess").SetZ(0, settings.event_process)


def settings_for_mdn(
    rastr_win: CDispatch,
    tras: float = 10.0,
    snap_path: str = "c:/tmp/",
    snap_max_count: int = 1,
    settings=Dynamic,
) -> None:
    """
    Выставляет рекомендованные настройки "МДН"

    :param rastr_win: Объект RastrWin3.
    :param tras: 10.0 секунд
    :param snap_path: 'c:/tmp/' или 'c:/tmp'
    :param snap_max_count: 1
    :param settings: Dynamic
    :return: None
    """
    logger.info("Выставляет рекомендованные настройки для определения МДН (Динамики)")
    tb_com_dynamics = rastr_win.Tables("com_dynamics")
    tb_com_dynamics.Cols("Tras").SetZ(0, tras)
    tb_com_dynamics.Cols("Hint").SetZ(0, settings.hint)
    tb_com_dynamics.Cols("Hmin").SetZ(0, settings.hmin)
    tb_com_dynamics.Cols("Hmax").SetZ(0, settings.hmax)
    tb_com_dynamics.Cols("Hout").SetZN(0, settings.hout)
    tb_com_dynamics.Cols("Mint").SetZN(0, settings.mint)
    tb_com_dynamics.Cols("SMint").SetZN(0, settings.smint)
    tb_com_dynamics.Cols("IntEpsilon").SetZN(0, settings.int_epsilon)
    tb_com_dynamics.Cols("InformOnStepChange").SetZ(0, settings.inform_on_step_change)
    tb_com_dynamics.Cols("Tf").SetZ(0, settings.tf)
    tb_com_dynamics.Cols("dEf").SetZ(0, settings.dEf)
    tb_com_dynamics.Cols("Npf").SetZ(0, settings.npf)
    tb_com_dynamics.Cols("Valid").SetZ(0, settings.valid)
    tb_com_dynamics.Cols("dempfrec").SetZ(0, settings.dempfrec)
    tb_com_dynamics.Cols("corrT").SetZ(0, settings.corrT)
    tb_com_dynamics.Cols("IsDemp").SetZ(0, settings.isDemp)
    tb_com_dynamics.Cols("frSXNtoY").SetZ(0, settings.frSXNtoY)
    tb_com_dynamics.Cols("SXNTolerance").SetZ(0, settings.sxn_tolerance)
    tb_com_dynamics.Cols("SnapPath").SetZ(0, snap_path)
    tb_com_dynamics.Cols("MaxResultFiles").SetZ(0, settings.max_result_files)
    tb_com_dynamics.Cols("SnapTemplate").SetZ(0, settings.snap_template)
    tb_com_dynamics.Cols("SnapAutoLoad").SetZ(0, settings.snap_auto_load)
    tb_com_dynamics.Cols("SnapMaxCount").SetZ(0, snap_max_count)
    tb_com_dynamics.Cols("TripGeneratorOnSpeed").SetZ(
        0, settings.trip_generator_on_speed
    )
    tb_com_dynamics.Cols("PickupDropout").SetZ(0, settings.pickup_dropout)
    tb_com_dynamics.Cols("RealtimeCSV").SetZ(0, settings.real_time_csv)
    tb_com_dynamics.Cols("PeriodAngle").SetZ(0, settings.period_angle)
    tb_com_dynamics.Cols("ResultFlowDirection").SetZ(0, settings.result_flow_direction)
    tb_com_dynamics.Cols("TreatWarningsAsErrors").SetZ(
        0, settings.treat_warnings_as_errors
    )
    tb_com_dynamics.Cols("EventProcess").SetZ(0, settings.event_process)
