"""

Настройки параметров эквивалентирования.

----------------------------------------------------------------
Навигация UI: Расчеты -> Параметры -> Тпблица: Эквивалент
----------------------------------------------------------------

- Отмеченные узла: (Отмеч) in code: (selekv) можно сохранить или эквивалентировать -
    {0 - Эквивалентировать, 1 - Сохранить}.

- Метод эквивалентирования: (Мет Экв) (met_ekv):
    0 - Гаус (Ward);
    1 - Димо;
    2 - М-д Жукова;
    3 - Гаус+Жуков;
    4 - Гаус+Димо;

        В Rastr реализован метод эквивалентирования – Гаусса (WARD) и два метода агрегации – Жукова и Димо.
    Агрегация - это объединение нескольких узлов в один.
        Эквивалентирование (агрегация) требует предварительно сбалансированного режима.
    Если режим несбалансирован, автоматически выполняется предварительная балансировка.
    Эквивалентирование не может быть выполнено для разошедшегося расчета
    режима. Эквивалентная схема становится текущей, замещает исходную и является сбалансированной.
    Режим для эквивалентирования должен быть получен без использвования опции «Удаление выключателей из схемы).

- Тип эквиваленитрования: (Тип Экв) in code: (tip_ekv) - {0 - Статическое; 1 - Динамическое}.

- Эквивалент узлов с фикс V: (фиксV) in code: (ekvgen) - {0 - Да; 1 - Нет};
    Можно разрешить или запретить эквивалентирование узлов с фиксированным модулем напряжения (даже если они отмечены).

- Тип эквивалентирования генераторов: (Тип Ген) in code: (tip_gen) - {0 - PQ; 1 - PQmin-Qmax; 2 - ЭДС за Хген};
    Один из трех возможных типов эквивалентирования узлов с фиксированным модулем напряжения:
        - PQ – пределы Qmin-Qmax и Vзд игнорируются;
        - PQmin-Qmax – пределы разносятся по узлам примыкания; в них фиксируется модуль напряжения;
        - ЭДС за Хген – в генераторах задается сопротивление (0.05 oe) в них фиксируется эдс.
    После эквивалентирования к узлам примыкания через эквивалентное сопротивление присоединются фиктивные узлы,
    в которых фиксируются модуль напряжения, и задаются Pген, Qмин-Qmax.

- Коэффициент для Xg_ген: (К_Х_Ген) in code: (kfc_x) - (по умолчанию 1) (type: float);
    Коэффициент для X_ген – поправочный коэффициент (по умолчанию 1) для расчета сопротивления генератора при
    типе "ЭДС за Хген".

- Учет потерь при разносе генерации: (dP_Ген) in code: (pot_gen) - {0 - Да, 1 - Нет};
    Да – потери на передачу мощности учитывают в эквивалентной мощности генерации, баланс генерации не сохраняется;
    Нет – потери на передачу учитываются в эквивалентной нагрузке, баланс генерации сохраняется (рекомендуется).

- Доля нагрузки, пересчитываемая в шунт: (d_наг) in code: (kpn):
    Доля нагрузки, пересчитываемая в шунт [0-1] и – задают долю нагрузки, пересчитываемой в шунт (α):
    0 – эквивалент в токах,
    1 – эквивалент в шунтах, при других значениях из диапазона [0-1] используется промежуточный способ;

- Учитывать СХН при эквивалентировании: (СХН) in code: (tip_sxn) - {0 - Да, 1 - Нет};
    Если Да – создаются эквивалентные СХН в узлах примыкания.

- "Умное" эквивалентирование: (Smart) in code: (smart) - {0 - Нет, 1 - Да};
    При использовании этой опции программа предварительно оценивает число новых,
    эквивалентных, ветвей, которые появляются при выполнении преобразования «звезда-многоугольник» для каждого узла.
    Т.е. узлы, эквивалентирование которых приводит к увеличению заполнения схемы, остаются в схеме.

- Удаление ветвей с сопротивлением большим: (Z_max) in code: (zmax): (type: float) рекомендуемое - 300.0;
    Позволяет удалять эквивалентные ветви с полным сопротивлением, большим заданного.
    Потоки мощности по удаляемым ветвям добавляются к нагрузке узла.

- Отмечать узлы после эквивалентирования: (Отм) in code: (otm_n) - {0 - Да, 1 - Нет}.
"""

# import Python lib
import logging

# import win32com
from win32com.client import CDispatch

logger = logging.getLogger("PyRastrWin")


def settings_for_equivalent(rastr_win: CDispatch) -> None:
    """
    Выставляет рекомендованные настройки "Эквивалентирования".
    :param rastr_win: Объект RastrWin3.
    :return: None
    """
    logger.info("Выставляет рекомендованные настройки Эквивалентирования")

    tb_com_ekviv = rastr_win.Tables("com_ekviv")

    # Отмеченные узла (Отмеч): {0 - Эквивалентировать, 1 - Сохранить}
    tb_com_ekviv.Cols("selekv").SetZ(0, 0)
    # Метод эквивалентирования (Мет Экв): {0 - Гаус (Ward); ...}
    tb_com_ekviv.Cols("met_ekv").SetZ(0, 0)
    # Тип эквиваленитрования (Тип Экв): {0 - Статическое; 1 - Динамическое}
    tb_com_ekviv.Cols("tip_ekv").SetZ(0, 0)
    # Эквивалент узлов с фикс V (фиксV): {0 - Да; 1 - Нет}
    tb_com_ekviv.Cols("ekvgen").SetZ(0, 0)
    # Тип эквивалентирования генераторов (Тип Ген): {..., 1 - PQmin-Qmax; ...}
    tb_com_ekviv.Cols("tip_gen").SetZ(0, 1)
    # Коэффициент для Xg_ген (К_Х_Ген): (по умолчанию 1) (type: float)
    tb_com_ekviv.Cols("kfc_x").SetZ(0, 0)
    # Учет потерь при разносе генерации dP_Ген: {0 - Да, 1 - Нет}
    tb_com_ekviv.Cols("pot_gen").SetZ(0, 0)
    # Доля нагрузки, пересчитываемая в шунт d_наг
    tb_com_ekviv.Cols("kpn").SetZ(0, 0)
    # Учитывать СХН при эквивалентировании (СХН): {0 - Да, 1 - Нет}
    tb_com_ekviv.Cols("tip_sxn").SetZ(0, 0)
    # "Умное" эквивалентирование (Smart): {0 - Нет, 1 - Да}
    tb_com_ekviv.Cols("smart").SetZ(0, 1)
    # Удаление ветвей с сопротивлением большим (Z_max): (type: float)
    tb_com_ekviv.Cols("zmax").SetZ(0, 300.0)
    # Отмечать узлы после эквивалентирования (Отм): {0 - Да, 1 - Нет}
    tb_com_ekviv.Cols("otm_n").SetZ(0, 1)
