"""
Настройки параметров расчета режима.

----------------------------------------------------------------
Навигация UI: Расчеты -> Параметры -> Тпблица: Режим
----------------------------------------------------------------

- Точность расчета (dP) in code (neb_p): Рекомендуется задавать ее в пределах [0,1–1] МВт;
    Расчет нормально заканчивается, если небаланс активной и реактивной мощности в каждом узле становится меньше заданной величины.

- Максимальное число итераций (It) in code (it_max): Рекомендуется задавать в пределах 20–50;
    Eсли число проделанных итераций достигает заданной величины, то расчет заканчивается аварийно с выдачей соответствующего сообщения.

- Стартовый алгоритм (Start) in code (start): {0-Да; 1-Нет}. По умолчанию задано «Да»;
    «Да» – для улучшения начального приближения перед запуском основного расчетного алгоритма используется стартовый алгоритм, основанный на методе Зейделя;
    «Нет» – стартовый алгоритм не используется.

- Плоский старт (Пл.старт) in code (flot): {0-Нет; 1-Да}. По умолчанию задано «Нет»;
    «Да» – расчет начинается с номинальных модулей и нулевых углов напряжений (в этом случае затраты времени на расчет могут стать большими);
    «Нет» – расчет начинается с текущего начального приближения.
    Если это приближение получено после аварийного завершения расчета, программа выдаст дополнительный запрос на расчет с плоского старта.

- Maкс. допустимое снижение V (dV-) in code (dv_min): Рекомендуется задавать в пределах [0.1–0.5] Uном;
  Максимально допустимое снижение напряжения по отношению к Uном.
  Если на входе расчета напряжение в каком-либо узле снизится больше указанной величины, то расчет аварийно завершится.
  При этом будет выдано соответствующее сообщение с номером узла, имеющего наименьшее напряжение.

- Maкс. допустимое превышение V (dV+) in code (dv_max): Рекомендуется задавать в пределах [2–5] Uном;
    Максимально допустимое превышение напряжения по отношению к Uном.
    Если в ходе расчета напряжение в каком-либо узле превысит указанную величину, то расчет аварийно завершится.
    При этом будет выдано соответствующее сообщение с номером узла, имеющего наибольшее напряжение.

- Maкс. допустимый угол по связи (dDelta) in code (dd_max): Рекомендуется задавать в пределах [90–170°];
    Максимально допустимый угол по ветви (в градусах).
    Если в ходе расчета угол по какой-либо линии превысит указанную величину, то расчет аварийно завершится.
    При этом будет выдано соответствующее сообщение c номером ветви, имеющей наибольший угол.

- Состояние расчета режима (Статус) in code (status): {0 - Нормально; 1 - Аварийно}.

- Учет частоты (W) in code (rr): {0 - Нет; 1 - Да}, по умолчанию – 0 - Нет;
    Да – расчет режима производится с учетом частоты (см. п.п. 2.13.3. «Расчет режима с учетом частоты»);
    Нет – расчет производится по стандартному алгоритму.

- Отклонение частоты (dF) in code (wt): в [о.е или % - ???] ∆𝑓 = (f−fном)⁄𝑓ном – отклонение частоты от номинальной.

- Пересчитывать P/Q узла по Pген (Ген->P) in code (gen_p): {0-Да; 1-Нет; 2-Только P; 3-Только Q}

- Пересчитывать P/Q нагрузки узла по таблице Нагрузки in code (load_p): {0-Да; 1-Нет}

- Метод Расчета (Метод) in code (method): {0-Ньютон; 1-Параметр}

- Метод учета ограничений Q (Метод учета ограничений Q) in code (method_ogr): {0-Стандартный; 1-Расширенный}

- Уровень печати (Печать) in code (print_mode): {0-Мин; 1-Макс; 2-Нет}

- Точный метод расчета Qmax(Qmax) in code (qmax): {0-Нет; 1-Да}. По умолчанию – Нет;
    Да – расчет Qmax для генераторов по заданным параметрам;
    Нет – расчет производится по стандартному алгоритму.
    Для расчета Qmax задание всех параметров обязательно (cм.раздел 2.13.2. Точный метод расчета Qmax),
    также не следует объединять генератор с повышающим трансформатором (эквивалентирование).

- Сопротивление выключателя (ое на 10^-6) in code (min_x): (type: float).

- Пересчет АТ/3х обм. трансофрматоров (Транс.) in code (calc_tr): {0-Нет; 1-Да; 2-Да (при изменении)}

- Пересчитывать (P/Q) нагрузки узла по ВРДО (Наг->P) in code (nag_p): {0-Нет; 1-Да}

- Удаление выключателей из схемы (Выкл) in code (rem_breaker): {0-Нет; 1-Да}

- Пересчет мощности генератора по ГРАМ (Грам) in code (gram): {0-Нет; 1-Да}

- Автоматическое создание БУ (БУ) in code (ctrl_baza): {0-Нет; 1-Да}

- Стартовый метод: число итераций (Z_it) in code (itz):

- Стартовый метод: Учет Qmax c итерации (Z_it_max) in code (itz_ogr_max): float

- Стартовый метод: Учет Qmin c итерации (Z_it_min) in code (itz_ogr_min): float

- Минимальное число узлов в остраве (Min_nodes) in code (min_nodes_in_island):

- Максимально допустимая крутизна СХН (Rslp) in code (max_slp_sxn):

- Минимально допустимая крутизна СХН (Rslp) in code (min_slp_sxn):
"""

# import Python
import logging

# import win32com
from win32com.client import CDispatch

logger = logging.getLogger("PyRastrWin")


def settings_for_regim(rastr_win: CDispatch) -> None:
    """
    Выставляет рекомендованные настройки "Режим".

    :param rastr_win: Объект RastrWin3.
    :returns: None
    """
    logger.info("Выставляет рекомендованные настройки для режима")
    tb_com_regim = rastr_win.Tables("com_regim")
    # Точность расчета (dP) Рекомендуется [0,1–1] МВт
    tb_com_regim.Cols("neb_p").SetZ(0, 0.1)
    # Максимальное число итераций (It) Рекомендуется 20–50
    tb_com_regim.Cols("it_max").SetZ(0, 50)
    # Стартовый алгоритм (Start) {0-Да; 1-Нет}
    tb_com_regim.Cols("start").SetZ(0, 0)
    # Плоский старт (Пл.старт) {0-Нет; 1-Да}
    tb_com_regim.Cols("flot").SetZ(0, 1)
    # Maкс. допустимое снижение V (dV-) Рекомендуется [0.1–0.5] Uном
    tb_com_regim.Cols("dv_min").SetZ(0, 0.5)
    # Maкс. допустимое превышение V (dV+) Рекомендуется [2–5] Uном
    tb_com_regim.Cols("dv_max").SetZ(0, 2.0)
    # Maкс. допустимый угол по связи (dDelta) Рекомендуется [90–170°]
    tb_com_regim.Cols("dd_max").SetZ(0, 170)
    # Состояние расчета режима (Статус) {0 - Нормально; 1 - Аварийно}
    tb_com_regim.Cols("status").SetZ(0, 0)
    # Учет частоты (W) {0 - Нет; 1 - Да}, по умолчанию–0
    tb_com_regim.Cols("rr").SetZ(0, 0)
    # отклонение частоты (dF)
    tb_com_regim.Cols("wt").SetZ(0, 0)
    # Пересчитывать P/Q узла по Pген (Ген->P) {0-Да;1-Нет;2-Только P;3-Только Q}
    tb_com_regim.Cols("gen_p").SetZ(0, 0)
    # Пересчитывать P/Q нагрузки узла по таблице Нагрузки {0-Да; 1-Нет}
    tb_com_regim.Cols("load_p").SetZ(0, 0)
    # Метод Расчета (Метод) {0-Ньютон; 1-Параметр}
    tb_com_regim.Cols("method").SetZ(0, 0)
    # Метод учета ограничений Q {0-Стандартный; 1-Расширенный}
    tb_com_regim.Cols("method_ogr").SetZ(0, 0)
    # Уровень печати (Печать) {0-Мин; 1-Макс; 2-Нет}
    tb_com_regim.Cols("print_mode").SetZ(0, 1)
    # Точный метод расчета Qmax(Qmax): {0-Нет; 1-Да}. По умолчанию – Нет
    tb_com_regim.Cols("qmax").SetZ(0, 0)
    # Сопротивление выключателя (ое на 10^-6)
    tb_com_regim.Cols("min_x").SetZ(0, 0)
    # Пересчет АТ/3х обм.транс-ов (Транс.): {0-Нет;1-Да;2-Да (при изменении)}
    tb_com_regim.Cols("calc_tr").SetZ(0, 0)
    # Пересчитывать (P/Q) нагрузки узла по ВРДО (Наг->P): {0-Нет; 1-Да}
    tb_com_regim.Cols("nag_p").SetZ(0, 0)
    # Удаление выключателей из схемы (Выкл): {0-Нет; 1-Да}
    tb_com_regim.Cols("rem_breaker").SetZ(0, 0)
    # Пересчет мощности генератора по ГРАМ (Грам): {0-Нет; 1-Да}
    tb_com_regim.Cols("gram").SetZ(0, 0)
    # Автоматическое создание БУ (БУ): {0-Нет; 1-Да}
    tb_com_regim.Cols("ctrl_baza").SetZ(0, 0)
    # Стартовый метод: число итераций (Z_it)
    tb_com_regim.Cols("itz").SetZ(0, 50)
    # Стартовый метод: Учет Qmax c итерации (Z_it_max)
    tb_com_regim.Cols("itz_ogr_max").SetZ(0, 0)
    # Стартовый метод: Учет Qmin c итерации (Z_it_min)
    tb_com_regim.Cols("itz_ogr_min").SetZ(0, 0)
    # Минимальное число узлов в остраве (Min_nodes)
    tb_com_regim.Cols("min_nodes_in_island").SetZ(0, 0)
    # Максимально допустимая крутизна СХН (Rslp)
    tb_com_regim.Cols("max_slp_sxn").SetZ(0, 500)
    # Минимально допустимая крутизна СХН (Rslp)
    tb_com_regim.Cols("min_slp_sxn").SetZ(0, -500)
